/**
 *  Interface to control LoggerFactories (LoggerFactory and related loggers) through
 *  ordinary console (in browsers).
 */
export interface LoggerControl {
    /**
     * Shows help for this object.
     */
    help(): void;
    /**
     * Lists all registered logger factories with associated log groups with respective ids
     */
    listFactories(): void;
    /**
     * Show settings for LoggerFactory id (see list() to get it) or null, for all.
     * @param idFactory LoggerFactory id
     */
    showSettings(idFactory: number | null): void;
    /**
     * Return LoggerFactoryControl object. Throws error when number is invalid.
     * @param idFactory Id factory
     * @returns {LoggerFactoryControl}
     */
    getLoggerFactoryControl(idFactory: number): LoggerFactoryControl;
}
/**
 * Interface to control LoggerFactory.
 */
export interface LoggerFactoryControl {
    /**
     * Shows help.
     */
    help(): void;
    /**
     * List all log groups registered to this factory.
     */
    listLogGroups(): void;
    /**
     * Set new log level for id of LogGroup, applied to both new and existing loggers. If id is null applies to all groups.
     * @param level Level must be one of: Trace, Debug, Info, Warn, Error or Fatal.
     * @param idGroup Group id, see listGroups() to find them or null for all groups.
     */
    setLogLevel(level: string, idGroup: number | null): void;
    /**
     * Set format for logging, format must be one of: "Default, YearMonthDayTime, YearDayMonthWithFullTime, YearDayMonthTime".
     * @param format One of Default, YearMonthDayTime, YearDayMonthWithFullTime, YearDayMonthTime.
     * @param idGroup Group id, see listGroups() or null to apply to all groups
     * @param showTimestamp True to show timestamp in log line
     * @param showLoggerName True to show logger name in log line
     */
    setLogFormat(format: string, showTimestamp: boolean, showLoggerName: boolean, idGroup: number | null): void;
    /**
     * Reset LogGroup back to original settings, applied to both new and existing loggers.
     * @param idGroup Group to reset, null for all groups.
     */
    reset(idGroup: number | null): void;
}
export declare class LoggerControlImpl implements LoggerControl {
    private static _help;
    help(): void;
    listFactories(): void;
    showSettings(id?: number | null): void;
    getLoggerFactoryControl(idFactory: number): LoggerFactoryControl;
    private static _getRuntimeSettingsLoggerFactories();
    private static _getSettings();
}
