import { LogLevel } from "../LoggerOptions";
import { Logger } from "./Logger";
import { LogGroupRule, LogGroupRuntimeSettings } from "./LoggerFactoryService";
export declare class Message {
    private _ready;
    private _logLevel;
    private _message;
    constructor(ready: boolean, logLevel: LogLevel, message?: string | null);
    ready: boolean;
    message: string | null;
    readonly logLevel: LogLevel;
}
/**
 * Abstract base logger, extend to easily implement a custom logger that
 * logs wherever you want. You only need to implement doLog(msg: string) and
 * log that somewhere (it will contain format and everything else).
 */
export declare abstract class AbstractLogger implements Logger {
    private _rule;
    private _level;
    private _allMessages;
    protected _name: string;
    protected _open: boolean;
    constructor(name: string, rule: LogGroupRule);
    readonly name: string;
    trace(msg: string, error?: Error | null): void;
    debug(msg: string, error?: Error | null): void;
    info(msg: string, error?: Error | null): void;
    warn(msg: string, error?: Error | null): void;
    error(msg: string, error?: Error | null): void;
    fatal(msg: string, error?: Error | null): void;
    tracec(msg: () => string, error?: () => Error | null): void;
    debugc(msg: () => string, error?: () => Error | null): void;
    infoc(msg: () => string, error?: () => Error | null): void;
    warnc(msg: () => string, error?: () => Error | null): void;
    errorc(msg: () => string, error?: () => Error | null): void;
    fatalc(msg: () => string, error?: () => Error | null): void;
    isTraceEnabled(): boolean;
    isDebugEnabled(): boolean;
    isInfoEnabled(): boolean;
    isWarnEnabled(): boolean;
    isErrorEnabled(): boolean;
    isFatalEnabled(): boolean;
    getLogLevel(): LogLevel;
    isOpen(): boolean;
    close(): void;
    protected abstract doLog(msg: string, logLevel: LogLevel): void;
    /**
     * Do not override by end user, will be gone in 0.3 release.
     */
    protected _addMessage(msg: Message): void;
    /**
     * Do not override by end user, will be gone in 0.3 release.
     */
    protected _log(level: LogLevel, msg: string, error?: Error | null): void;
    /**
     * Do not override by end user, will be gone in 0.3 release.
     */
    protected _logc(level: LogLevel, msg: () => string, error?: () => Error | null): void;
    /**
     * Do not override by end user, will be gone in 0.3 release.
     */
    protected createMessage(level: LogLevel, msg: string, date: Date, error?: Error | null): Message;
    /**
     * Do not override by end user, will be gone in 0.3 release.
     */
    protected processMessages(): void;
}
/**
 * Future base logger, temporal name will replace AbstractLogger in future 0.3 release.
 */
export declare abstract class AbstractBaseLogger extends AbstractLogger {
    private _logGroupRuntimeSettings;
    constructor(name: string, logGroupRuntimeSettings: LogGroupRuntimeSettings);
    /**
     * Do not override by end user, will be gone in 0.3 release.
     */
    protected _log(level: LogLevel, msg: string, error?: Error | null): void;
    /**
     * Do not override by end user, will be gone in 0.3 release.
     */
    protected _logc(level: LogLevel, msg: () => string, error?: () => Error | null): void;
    /**
     * Do not override by end user, will be gone in 0.3 release.
     */
    protected createMessage(level: LogLevel, msg: string, date: Date, error?: Error | null): Message;
}
/**
 * Simple logger, that logs to the console. If the console is unavailable will throw exception.
 */
export declare class ConsoleLoggerImpl extends AbstractBaseLogger {
    constructor(name: string, logGroupRuntimeSettings: LogGroupRuntimeSettings);
    protected doLog(msg: string, logLevel: LogLevel): void;
}
/**
 * Logger which buffers all messages, use with care due to possible high memory footprint.
 * Can be convenient in some cases. Call toString() for full output, or cast to this class
 * and call getMessages() to do something with it yourself.
 */
export declare class MessageBufferLoggerImpl extends AbstractBaseLogger {
    private messages;
    constructor(name: string, logGroupRuntimeSettings: LogGroupRuntimeSettings);
    close(): void;
    getMessages(): string[];
    toString(): string;
    protected doLog(msg: string, logLevel: LogLevel): void;
}
